@@ -5,17 +5,21 @@
 #include "daemon/bindings/impl.h"
 
 #include "contrib/base64.h"
+#include "contrib/cleanup.h"
 #include "daemon/network.h"
 #include "daemon/tls.h"
+#include "lib/utils.h"
 
 #include <stdlib.h>
 
+#define PROXY_DATA_STRLEN (INET6_ADDRSTRLEN + 1 + 3 + 1)
+
 /** Table and next index on top of stack -> append entries for given endpoint_array_t. */
-static int net_list_add(const char *key, void *val, void *ext)
+static int net_list_add(const char *b_key, uint32_t key_len, trie_val_t *val, void *ext)
 {
+	endpoint_array_t *ep_array = *val;
 	lua_State *L = (lua_State *)ext;
 	lua_Integer i = lua_tointeger(L, -1);
-	endpoint_array_t *ep_array = val;
 	for (int j = 0; j < ep_array->len; ++j) {
 		struct endpoint *ep = &ep_array->at[j];
 		lua_newtable(L);  // connection tuple
@@ -54,7 +58,15 @@ static int net_list_add(const char *key, void *val, void *ext)
 		}
 		lua_setfield(L, -2, "family");
 
-		lua_pushstring(L, key);
+		const char *ip_str_const = network_endpoint_key_str((struct endpoint_key *) b_key);
+		kr_require(ip_str_const);
+		auto_free char *ip_str = strdup(ip_str_const);
+		kr_require(ip_str);
+		char *hm = strchr(ip_str, '#');
+		if (hm) /* Omit port */
+			*hm = '\0';
+		lua_pushstring(L, ip_str);
+
 		if (ep->family == AF_INET || ep->family == AF_INET6) {
 			lua_setfield(L, -2, "ip");
 			lua_pushboolean(L, ep->flags.freebind);
@@ -98,7 +110,7 @@ static int net_list(lua_State *L)
 {
 	lua_newtable(L);
 	lua_pushinteger(L, 1);
-	map_walk(&the_worker->engine->net.endpoints, net_list_add, L);
+	trie_apply_with_key(the_worker->engine->net.endpoints, net_list_add, L);
 	lua_pop(L, 1);
 	return 1;
 }
@@ -280,6 +292,114 @@ static int net_listen(lua_State *L)
 	return 1;
 }
 
+/** Prints the specified `data` into the specified `dst` buffer. */
+static char *proxy_data_to_string(int af, const struct net_proxy_data *data,
+		char *dst, size_t size)
+{
+	kr_assert(size >= PROXY_DATA_STRLEN);
+	const void *in_addr = (af == AF_INET)
+		? (void *) &data->addr.ip4
+		: (void *) &data->addr.ip6;
+	char *cur = dst;
+
+	const char *ret = inet_ntop(af, in_addr, cur, size);
+	if (!ret)
+		return NULL;
+
+	cur += strlen(cur); /*< advance cursor to after the address */
+	*(cur++) = '/';
+	int masklen = snprintf(cur, 3 + 1, "%u", data->netmask);
+	cur[masklen] = '\0';
+	return dst;
+}
+
+/** Put all IP addresses from `trie` into the table at the top of the Lua stack.
+ * For each address, increment the integer at `i`. All addresses in `trie` must
+ * be from the specified `family`. */
+static void net_proxy_addr_put(lua_State *L, int family, trie_t *trie, int *i)
+{
+	char addrbuf[PROXY_DATA_STRLEN];
+	const char *addr;
+	trie_it_t *it;
+	for (it = trie_it_begin(trie); !trie_it_finished(it); trie_it_next(it)) {
+		lua_pushinteger(L, *i);
+		struct net_proxy_data *data = *trie_it_val(it);
+		addr = proxy_data_to_string(family, data,
+				addrbuf, sizeof(addrbuf));
+		lua_pushstring(L, addr);
+		lua_settable(L, -3);
+		*i += 1;
+	}
+	trie_it_free(it);
+}
+
+/** Allow PROXYv2 headers for IP address. */
+static int net_proxy_allowed(lua_State *L)
+{
+	struct network *net = &the_worker->engine->net;
+	int n = lua_gettop(L);
+	int i = 1;
+	const char *addr;
+
+	/* Return current state */
+	if (n == 0) {
+		lua_newtable(L);
+		i = 1;
+
+		if (net->proxy_all4) {
+			lua_pushinteger(L, i);
+			lua_pushstring(L, "0.0.0.0/0");
+			lua_settable(L, -3);
+			i += 1;
+		} else {
+			net_proxy_addr_put(L, AF_INET, net->proxy_addrs4, &i);
+		}
+
+		if (net->proxy_all6) {
+			lua_pushinteger(L, i);
+			lua_pushstring(L, "::/0");
+			lua_settable(L, -3);
+			i += 1;
+		} else {
+			net_proxy_addr_put(L, AF_INET6, net->proxy_addrs6, &i);
+		}
+
+		return 1;
+	}
+
+	if (n != 1)
+		lua_error_p(L, "net.proxy_allowed() takes one parameter (string or table)");
+
+	if (!lua_istable(L, 1) && !lua_isstring(L, 1))
+		lua_error_p(L, "net.proxy_allowed() argument must be string or table");
+
+	/* Reset allowed proxy addresses */
+	network_proxy_reset(net);
+
+	/* Add new proxy addresses */
+	if (lua_istable(L, 1)) {
+		for (i = 1; !lua_isnil(L, -1); i++) {
+			lua_pushinteger(L, i);
+			lua_gettable(L, 1);
+			if (lua_isnil(L, -1)) /* missing value - end iteration */
+				break;
+			if (!lua_isstring(L, -1))
+				lua_error_p(L, "net.proxy_allowed() argument may only contain strings");
+			addr = lua_tostring(L, -1);
+			int ret = network_proxy_allow(net, addr);
+			if (ret)
+				lua_error_p(L, "invalid argument");
+		}
+	} else if (lua_isstring(L, 1)) {
+		addr = lua_tostring(L, 1);
+		int ret = network_proxy_allow(net, addr);
+		if (ret)
+			lua_error_p(L, "invalid argument");
+	}
+
+	return 0;
+}
+
 /** Close endpoint. */
 static int net_close(lua_State *L)
 {
@@ -333,7 +453,17 @@ static int net_interfaces(lua_State *L)
 		} else {
 			buf[0] = '\0';
 		}
-		lua_pushstring(L, buf);
+
+		if (kr_sockaddr_link_local((struct sockaddr *) &iface.address)) {
+			/* Link-local IPv6: add %interface prefix */
+			auto_free char *str = NULL;
+			int ret = asprintf(&str, "%s%%%s", buf, iface.name);
+			kr_assert(ret > 0);
+			lua_pushstring(L, str);
+		} else {
+			lua_pushstring(L, buf);
+		}
+
 		lua_rawseti(L, -2, lua_objlen(L, -2) + 1);
 		lua_setfield(L, -2, "addr");
 
@@ -929,11 +1059,11 @@ static int net_tls_sticket_secret_file(lua_State *L)
 
 static int net_outgoing(lua_State *L, int family)
 {
-	union inaddr *addr;
+	union kr_sockaddr *addr;
 	if (family == AF_INET)
-		addr = (union inaddr*)&the_worker->out_addr4;
+		addr = (union kr_sockaddr*)&the_worker->out_addr4;
 	else
-		addr = (union inaddr*)&the_worker->out_addr6;
+		addr = (union kr_sockaddr*)&the_worker->out_addr6;
 
 	if (lua_gettop(L) == 0) { /* Return the current value. */
 		if (addr->ip.sa_family == AF_UNSPEC) {
@@ -1102,6 +1232,7 @@ int kr_bindings_net(lua_State *L)
 	static const luaL_Reg lib[] = {
 		{ "list",         net_list },
 		{ "listen",       net_listen },
+		{ "proxy_allowed", net_proxy_allowed },
 		{ "close",        net_close },
 		{ "interfaces",   net_interfaces },
 		{ "bufsize",      net_bufsize },
