@@ -148,6 +148,8 @@ void udp_recv(uv_udp_t *handle, ssize_t nread, const uv_buf_t *buf,
 
 static int family_to_freebind_option(sa_family_t sa_family, int *level, int *name)
 {
+#define LOG_NO_FB kr_log_error(NETWORK, "your system does not support 'freebind', " \
+				"please remove it from your configuration\n")
 	switch (sa_family) {
 	case AF_INET:
 		*level = IPPROTO_IP;
@@ -156,6 +158,7 @@ static int family_to_freebind_option(sa_family_t sa_family, int *level, int *nam
 #elif defined(IP_BINDANY)
 		*name = IP_BINDANY;
 #else
+		LOG_NO_FB;
 		return kr_error(ENOTSUP);
 #endif
 		break;
@@ -167,6 +170,7 @@ static int family_to_freebind_option(sa_family_t sa_family, int *level, int *nam
 		*level = IPPROTO_IPV6;
 		*name = IPV6_BINDANY;
 #else
+		LOG_NO_FB;
 		return kr_error(ENOTSUP);
 #endif
 		break;
@@ -1029,15 +1033,24 @@ int io_listen_xdp(uv_loop_t *loop, struct endpoint *ep, const char *ifname)
 	xdp_handle_data_t *xhd = malloc(sizeof(*xhd));
 	if (!xhd) return kr_error(ENOMEM);
 
-	const int port = ep->port ? ep->port : // all ports otherwise
-			#if KNOT_VERSION_HEX >= 0x030100
-				(KNOT_XDP_LISTEN_PORT_PASS | 0);
-			#else
-				KNOT_XDP_LISTEN_PORT_ALL;
-			#endif
 	xhd->socket = NULL; // needed for some reason
-	int ret = knot_xdp_init(&xhd->socket, ifname, ep->nic_queue, port,
-				KNOT_XDP_LOAD_BPF_MAYBE);
+
+	// This call is a libknot version hell, unfortunately.
+	int ret = knot_xdp_init(&xhd->socket, ifname, ep->nic_queue,
+		#if KNOT_VERSION_HEX < 0x030100
+			ep->port ? ep->port : KNOT_XDP_LISTEN_PORT_ALL,
+			KNOT_XDP_LOAD_BPF_MAYBE
+		#elif KNOT_VERSION_HEX < 0x030200
+			ep->port ? ep->port : (KNOT_XDP_LISTEN_PORT_PASS | 0),
+			KNOT_XDP_LOAD_BPF_MAYBE
+		#else
+			KNOT_XDP_FILTER_UDP | (ep->port ? 0 : KNOT_XDP_FILTER_PASS),
+			ep->port, 0/*quic_port*/,
+			KNOT_XDP_LOAD_BPF_MAYBE,
+			NULL/*xdp_config*/
+		#endif
+		);
+
 	if (!ret) xdp_warn_mode(ifname);
 
 	if (!ret) ret = uv_idle_init(loop, &xhd->tx_waker);
