@@ -5,6 +5,8 @@
 #include "lib/generic/queue.h"
 #include <string.h>
 
+extern inline void * queue_head_impl(const struct queue *q);
+
 void queue_init_impl(struct queue *q, size_t item_size)
 {
 	q->len = 0;
@@ -36,8 +38,9 @@ void queue_deinit_impl(struct queue *q)
 
 static struct queue_chunk * queue_chunk_new(const struct queue *q)
 {
+	/* size_t cast is to avoid unintended sign-extension */
 	struct queue_chunk *c = malloc(offsetof(struct queue_chunk, data)
-					+ q->chunk_cap * q->item_size);
+					+ (size_t) q->chunk_cap * (size_t) q->item_size);
 	if (unlikely(!c)) abort(); // simplify stuff
 	memset(c, 0, offsetof(struct queue_chunk, data));
 	c->cap = q->chunk_cap;
@@ -57,9 +60,10 @@ void * queue_push_impl(struct queue *q)
 	} else
 	if (t->end == t->cap) {
 		if (t->begin * 2 >= t->cap) {
-			/* Utilization is below 50%, so let's shift (no overlap). */
+			/* Utilization is below 50%, so let's shift (no overlap).
+			 * (size_t cast is to avoid unintended sign-extension) */
 			memcpy(t->data, t->data + t->begin * q->item_size,
-				(t->end - t->begin) * q->item_size);
+				(size_t) (t->end - t->begin) * (size_t) q->item_size);
 			t->end -= t->begin;
 			t->begin = 0;
 		} else {
@@ -91,10 +95,11 @@ void * queue_push_head_impl(struct queue *q)
 	if (h->begin == 0) {
 		if (h->end * 2 <= h->cap) {
 			/* Utilization is below 50%, so let's shift (no overlap).
-			 * Computations here are simplified due to h->begin == 0. */
+			 * Computations here are simplified due to h->begin == 0.
+			 * (size_t cast is to avoid unintended sign-extension) */
 			const int cnt = h->end;
 			memcpy(h->data + (h->cap - cnt) * q->item_size, h->data,
-				cnt * q->item_size);
+				(size_t) cnt * (size_t) q->item_size);
 			h->begin = h->cap - cnt;
 			h->end = h->cap;
 		} else {
@@ -111,3 +116,25 @@ void * queue_push_head_impl(struct queue *q)
 	return h->data + q->item_size * h->begin;
 }
 
+void queue_pop_impl(struct queue *q)
+{
+	kr_require(q);
+	struct queue_chunk *h = q->head;
+	kr_require(h && h->end > h->begin);
+	if (h->end - h->begin == 1) {
+		/* removing the last element in the chunk */
+		kr_require((q->len == 1) == (q->head == q->tail));
+		if (q->len == 1) {
+			q->tail = NULL;
+			kr_require(!h->next);
+		} else {
+			kr_require(h->next);
+		}
+		q->head = h->next;
+		free(h);
+	} else {
+		++(h->begin);
+	}
+	--(q->len);
+}
+
