@@ -36,7 +36,8 @@ static void *load_symbol(void *lib, const char *prefix, const char *name)
 
 static int load_library(struct kr_module *module, const char *name, const char *path)
 {
-	assert(module && name && path);
+	if (kr_fails_assert(module && name && path))
+		return kr_error(EINVAL);
 	/* Absolute or relative path (then only library search path is used). */
 	auto_free char *lib_path = kr_strcatdup(4, path, "/", name, LIBEXT);
 	if (lib_path == NULL) {
@@ -86,7 +87,7 @@ static int load_sym_c(struct kr_module *module, uint32_t api_required)
 	    || load_symbol(module->lib, m_prefix, "props")) {
 		/* In case someone re-compiled against new kresd
 		 * but haven't actually changed the symbols. */
-		kr_log_error("[system] module %s requires upgrade.  Please refer to "
+		kr_log_error(SYSTEM, "module %s requires upgrade.  Please refer to "
 			"https://knot-resolver.readthedocs.io/en/stable/upgrading.html",
 			module->name);
 		return kr_error(ENOTSUP);
