@@ -813,7 +813,7 @@ int kr_resolve_consume(struct kr_request *request, struct kr_transport **transpo
 		return KR_STATE_PRODUCE;
 
 	/* Packet cleared, derandomize QNAME. */
-	knot_dname_t *qname_raw = knot_pkt_qname(packet);
+	knot_dname_t *qname_raw = kr_pkt_qname_raw(packet);
 	if (qname_raw && qry->secret != 0) {
 		randomized_qname_case(qname_raw, qry->secret);
 	}
@@ -1425,7 +1425,7 @@ int kr_resolve_produce(struct kr_request *request, struct kr_transport **transpo
 
 	/* Randomize query case (if not in not turned off) */
 	qry->secret = qry->flags.NO_0X20 ? 0 : kr_rand_bytes(sizeof(qry->secret));
-	knot_dname_t *qname_raw = knot_pkt_qname(packet);
+	knot_dname_t *qname_raw = kr_pkt_qname_raw(packet);
 	randomized_qname_case(qname_raw, qry->secret);
 
 	/*
@@ -1529,9 +1529,9 @@ int kr_resolve_checkout(struct kr_request *request, const struct sockaddr *src,
 	}
 
 	/* Randomize query case (if secret changed) */
-	knot_dname_t *qname = knot_pkt_qname(packet);
+	knot_dname_t *qname_raw = kr_pkt_qname_raw(packet);
 	if (qry->secret != old_minimization_secret) {
-		randomized_qname_case(qname, qry->secret);
+		randomized_qname_case(qname_raw, qry->secret);
 	}
 
 	/* Write down OPT unless in safemode */
