@@ -7,7 +7,7 @@
 
 static void test_module_params(void **state)
 {
-	struct kr_module module;
+	struct kr_module module = { 0 };
 	assert_int_equal(kr_module_load(NULL, NULL, NULL), kr_error(EINVAL));
 	assert_int_equal(kr_module_load(&module, NULL, NULL), kr_error(EINVAL));
 	kr_module_unload(NULL);
@@ -15,14 +15,14 @@ static void test_module_params(void **state)
 
 static void test_module_builtin(void **state)
 {
-	struct kr_module module;
+	struct kr_module module = { 0 };
 	assert_int_equal(kr_module_load(&module, "iterate", NULL), 0);
 	kr_module_unload(&module);
 }
 
 static void test_module_c(void **state)
 {
-	struct kr_module module;
+	struct kr_module module = { 0 };
 	assert_int_equal(kr_module_load(&module, "mock_cmodule", "tests/unit"), 0);
 	kr_module_unload(&module);
 }
